/*
** lcd.c
**
** LCD 3310 driver
** Unbuffered version - very small memory footprint
** Target: ATMEGA128 :: AVR-GCC
**
** Written by Tony Myatt - 2007
** Quantum Torque - www.quantumtorque.com
*/
#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "lcd.h"

/* delay macro function */
#define lcd_delay() for(i=-32000;i<32000;i++)
#define NUM_PIX 4
#define limit_X	96
#define limit_Y	9

/* Command type sent to the lcd */
typedef enum { 
	LCD_CMD  = 0, 
	LCD_DATA = 1 
} LcdCmdData;

/* Function prototypes */
void lcd_base_addr(unsigned int addr);
void lcd_send(unsigned char data, LcdCmdData cd);

/* The lcd cursor position */
int lcdCacheIdx;


unsigned char ecran[9][96];	
/* piesa actuala */
int piece;

/* retin pixelii care compun piesele */
int my_x[NUM_PIX], my_y[NUM_PIX];
int x[NUM_PIX], y[NUM_PIX];

/* Performs IO & LCD controller initialization */
void lcd_init(void)
{
	int i;
	
	BUTTON_DDR &= (!(B1 | B2 | B3 | B4 | B5 | B6));
	BUTTON_PORT |= (B1 | B2 | B3 | B4 | B5 | B6);
    
	// Pull-up on reset pin
    LCD_PORT |= LCD_RST_PIN;
	
	// Set output bits on lcd port
	LCD_DDR |= LCD_RST_PIN | LCD_CE_PIN | LCD_DC_PIN | LCD_DATA_PIN | LCD_CLK_PIN;
    
	// Wait after VCC high for reset (max 30ms)
    _delay_ms(15);
    
    // Toggle display reset pin
    LCD_PORT &= ~LCD_RST_PIN;
    lcd_delay();
    LCD_PORT |= LCD_RST_PIN;

    // Disable LCD controller
    LCD_PORT |= LCD_CE_PIN;

	lcd_send(0x21, LCD_CMD);
	lcd_send(0xC8, LCD_CMD);
	lcd_send(0x06, LCD_CMD);
	lcd_send(0x13, LCD_CMD);
	lcd_send(0x20, LCD_CMD);
	lcd_send(0x0C, LCD_CMD);
	
	lcd_clear();
	
	//fdevopen(lcd_chr, 0);
}

void init_player() {
	my_x[0] = 14;
		my_x[1] = 14;
		my_x[2] = 14;
		my_x[3] = 14;
		my_y[0] = NUM_PIX - 4;
		my_y[1] = NUM_PIX - 3;
		my_y[2] = NUM_PIX - 2;
		my_y[3] = NUM_PIX - 1;	
}
/* Sterge elementele din matrice */
void ecran_clear()
{
	int i, j;
	for(i = 0; i < limit_X; i++)
		for(j = 0; j < limit_Y; j++)
			ecran[j][i] = 0;
}

/* Functia ecran_init sterge continutul ecranului,
 afiseaza mesajul "PRESS BUTTON 1"
 */
void ecran_init()
{
	int i, j;
	
	for(i = 0; i < limit_X; i++)
		for(j = 0; j < limit_Y; j++)
			ecran[j][i] = 0;
			
	
	i = 25;
	j = 3;
	
	ecran[j][i++] = 0x7F; //P
	ecran[j][i++] = 0x09;
	ecran[j][i++] = 0x09;
	ecran[j][i++] = 0x09;
	ecran[j][i++] = 0x06;
	
	ecran[j][i++] = 0x7F; //R
	ecran[j][i++] = 0x09;
	ecran[j][i++] = 0x19;
	ecran[j][i++] = 0x29;
	ecran[j][i++] = 0x46;
	
	ecran[j][i++] = 0x7F; //E
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x41;
	
	ecran[j][i++] = 0x46;	//S
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x31;
	
	ecran[j][i++] = 0x46;	//S
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x31;
	
	ecran[j][i++] = 0x00; //' '
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;
	
	ecran[j][i++] = 0x7F; //B
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;	
	ecran[j][i++] = 0x49;	
	ecran[j][i++] = 0x36;
 
	ecran[j][i++] = 0x3F; //U
	ecran[j][i++] = 0x40;
	ecran[j][i++] = 0x40;	
	ecran[j][i++] = 0x40;	
	ecran[j][i++] = 0x3F;

	ecran[j][i++] = 0x01;	//T
	ecran[j][i++] = 0x01;
	ecran[j][i++] = 0x7F;
	ecran[j][i++] = 0x01;
	ecran[j][i++] = 0x01;
	
	ecran[j][i++] = 0x01;	//T
	ecran[j][i++] = 0x01;
	ecran[j][i++] = 0x7F;
	ecran[j][i++] = 0x01;
	ecran[j][i++] = 0x01;
	
	ecran[j][i++] = 0x3E; //O
	ecran[j][i++] = 0x41;
	ecran[j][i++] = 0x41;
	ecran[j][i++] = 0x41;
	ecran[j][i++] = 0x3E;		
   
    ecran[j][i++] = 0x7F; //N
	ecran[j][i++] = 0x04;
	ecran[j][i++] = 0x08;	
	ecran[j][i++] = 0x10;	
	ecran[j][i++] = 0x7F;

	ecran[j][i++] = 0x00; //' '
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;

	ecran[j][i++] = 0x00; //1
	ecran[j][i++] = 0x42;
	ecran[j][i++] = 0x7F;
	ecran[j][i++] = 0x40;
	ecran[j][i++] = 0x00;
	
}

void ecran_end_game()
{
	int i, j;
	
	for(i = 0; i < limit_X; i++)
		for(j = 0; j < limit_Y; j++)
			ecran[j][i] = 0;
			
	i = 35;
	j = 3;
	
	ecran[j][i++] = 0x7F; //E
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;	
	ecran[j][i++] = 0x49;	
	ecran[j][i++] = 0x41;
	
	ecran[j][i++] = 0x7F; //N
	ecran[j][i++] = 0x04;
	ecran[j][i++] = 0x08;	
	ecran[j][i++] = 0x10;	
	ecran[j][i++] = 0x7F;

	ecran[j][i++] = 0x7F; //D
	ecran[j][i++] = 0x41;
	ecran[j][i++] = 0x41;	
	ecran[j][i++] = 0x22;	
	ecran[j][i++] = 0x1C;
	
	ecran[j][i++] = 0x00; //' '
	ecran[j][i++] = 0x00;
	ecran[j][i++] = 0x00;	
	ecran[j][i++] = 0x00;	
	ecran[j][i++] = 0x00;
	 
	ecran[j][i++] = 0x3E; //G
	ecran[j][i++] = 0x41;
	ecran[j][i++] = 0x49;	
	ecran[j][i++] = 0x49;	
	ecran[j][i++] = 0x7A;

	ecran[j][i++] = 0x7F; //A
	ecran[j][i++] = 0x11;
	ecran[j][i++] = 0x11;	
	ecran[j][i++] = 0x11;	
	ecran[j][i++] = 0x7E;
	
	ecran[j][i++] = 0x7F; //M
	ecran[j][i++] = 0x02;
	ecran[j][i++] = 0x0C;	
	ecran[j][i++] = 0x02;	
	ecran[j][i++] = 0x7F;

	ecran[j][i++] = 0x7F; //E
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x49;
	ecran[j][i++] = 0x41;

}

void lcd_base_addr(unsigned int addr) {
	lcd_send(0x80 |(addr % LCD_X_RES), LCD_CMD);
	lcd_send(0x40 |(addr / LCD_X_RES), LCD_CMD);
}


/* Functia  afiseaza pe ecran*/
void display()
{
	int i, j;
	
	lcd_send(0x80, LCD_CMD);
	lcd_send(0x40, LCD_CMD);
	
	for(j = 0; j < limit_Y; j++)
		for(i = 0; i < limit_X; i++) {
			lcd_send(ecran[j][i], LCD_DATA);
		}
}

/* Clears the display */
void lcd_clear(void)
{
	int i;
	
	lcd_send(0x80, LCD_CMD);
	lcd_send(0x40, LCD_CMD);	
	
    // Set the entire cache to zero and write 0s to lcd
    for(i = 0; i<96 * 9;i++)
	{
		lcd_send(0x00, LCD_DATA);
    }

}

/* Sends data to display controller */
void lcd_send(unsigned char data, LcdCmdData cd)
{

	LCD_DDR |= LCD_DATA_PIN | LCD_DC_PIN;

	LCD_PORT &= ~LCD_CE_PIN;


    if(cd == LCD_DATA)
	{
        LCD_PORT |= LCD_DC_PIN;
    }
	else
	{
        LCD_PORT &= ~(LCD_DC_PIN);
    }	
    // Enable display controller (active low)
    
    // Either command or data

	
	unsigned char i;
	for(i=0;i<8;i++)
	{
	
		// Set the DATA pin value
		if((data>>(7-i)) & 0x01)
		{
			LCD_PORT |= LCD_DATA_PIN;
		}
		else
		{
			LCD_PORT &= ~LCD_DATA_PIN;
		}
		
		// Toggle the clock
		LCD_PORT |= LCD_CLK_PIN;
		LCD_PORT &= ~LCD_CLK_PIN;
	}

	// Disable display controller
    LCD_PORT |= LCD_CE_PIN;	
	
	LCD_DDR &= ~(LCD_DATA_PIN | LCD_DC_PIN);
	LCD_PORT |= LCD_DATA_PIN | LCD_DC_PIN;
}


/* Functia isSetS verifica daca pixelul (x,y) este setat deja. */
int isSetS(int x, int y)
{
	if(y % 2 == 0)
	{
		if((ecran[y / 2][x * 4] & 0x0F) == 0)
			return 0;
		return 1;
	}
	else
	{
		if((ecran[y / 2][x * 4] & 0xF0) == 0)
			return 0;
		return 1;
	}
}

/* Functia isSet verifica daca pixelul (x,y) este ocupat de o alta piesa. */
int isSet(int xx, int yy)
{	
	int i;
	for(i = 0; i < 4; i++)
		if(x[i] == xx && y[i] == yy)
			return 0;
		
	if (xx < 0 || yy < 0 ||  xx > 23 || yy > 15)
		return 1;
	
	if(yy % 2 == 0)
	{
		if((ecran[yy / 2][xx * 4] & 0x0F) == 0)
			return 0;
		return 1;
	}
	else
	{
		if((ecran[yy / 2][xx * 4] & 0xF0) == 0)
			return 0;
		return 1;
	}
}

int good_move(int dx, int dy)
{
	int i;
	for(i = 0; i < 4; i++)
	{
		if(x[i] + dx < 0 || x[i] + dx > 23 ||  y[i] + dy < 0 || y[i] +dy > 15)
			return 0;
	}
	
	return 1;
}

int collision(int dx, int dy) {
	int i;
	for(i = 0; i < 4; i++)
	{
		if(isSet(x[i] + dx, y[i] + dy))
			return 0;
	}
	return 1;
}

void get_car(int *t_x, int *t_y) {
	int i;
	if(t_x == NULL || t_y == NULL) {
		t_x = x;
		t_y = y;
	}
	for(i = 0; i < 4; i++) {
		if(y[i] % 2 == 1) {
			ecran[t_y[i] / 2][t_x[i] * 4] |= 0xF0;
			ecran[t_y[i] / 2][t_x[i] * 4 + 1] |= 0xF0;
			ecran[t_y[i] / 2][t_x[i] * 4 + 2] |= 0xF0;
			ecran[t_y[i] / 2][t_x[i] * 4 + 3] |= 0xF0;		
		}
		else {
			ecran[t_y[i] / 2][t_x[i] * 4] |= 0x0F;
			ecran[t_y[i] / 2][t_x[i] * 4 + 1]|= 0x0F;
			ecran[t_y[i] / 2][t_x[i] * 4 + 2] |= 0x0F;
			ecran[t_y[i] / 2][t_x[i] * 4 + 3] |= 0x0F;	
		}
	}
}


void del_car(int *x, int *y) {
	int i;
	for(i = 0; i < 4; i++) {
		if(y[i] % 2 == 0) {
			ecran[y[i] / 2][x[i] * 4] &= 0xF0;
			ecran[y[i] / 2][x[i] * 4 + 1] &= 0xF0;
			ecran[y[i] / 2][x[i] * 4 + 2] &= 0xF0;
			ecran[y[i] / 2][x[i] * 4 + 3] &= 0xF0;		
		}
		else {
			ecran[y[i] / 2][x[i] * 4] &= 0x0F;
			ecran[y[i] / 2][x[i] * 4 + 1]&= 0x0F;
			ecran[y[i] / 2][x[i] * 4 + 2] &= 0x0F;
			ecran[y[i] / 2][x[i] * 4 + 3] &= 0x0F;	
		}
	}

}

/* Functii de mutare a piesei pe cele 3 directii : jos, dreapta, stanga */
int go_down() {

	int i;
	if (!good_move(0, 1)) {
		del_car(x, y);				
		return 0;
	}
	
	for(i = 0; i < 4; i++)
	{
		y[i] += 1;
	}
	
	get_car(x, y);
	return 1;
	
}

int go_right() {

	int i;
	if (!good_move(1, 0))
		return 0;
	
	del_car(my_x, my_y);
	for(i = 0; i < 4; i++)
	{
		my_x[i] += 1;
	}
	
	get_car(my_x, my_y);
	return 1;

}

int go_left() {

	int i;
	if (!good_move(-1, 0))
		return 0;
		
	del_car(my_x, my_y);
	for(i = 0; i < 4; i++)
	{
		my_x[i] += -1;
	}
	
	get_car(my_x, my_y);
	return 1;


}

/* Functii de creare a masinilor. */
 
void create_car() {

	x[0] = 14;
	x[1] = 14;
	x[2] = 14;
	x[3] = 14;
	y[0] = 0;
	y[1] = 1;
	y[2] = 2;
	y[3] = 3;	

}
